@@ -1,5 +1,5 @@
-from typing import Any, Callable, Dict, List, Optional, Pattern, Tuple, Union, overload
-from unittest import mock
+from typing import Callable, Dict, List, Optional, Tuple, Union, overload
+from warnings import warn
 
 from httpcore import (
     AsyncByteStream,
@@ -11,17 +11,19 @@ from httpcore import (
 from .models import (
     URL,
     AsyncResponse,
+    CallList,
     ContentDataTypes,
     DefaultType,
     Headers,
     HeaderTypes,
+    JSONTypes,
+    QueryParamTypes,
     Request,
     RequestPattern,
     Response,
     ResponseTemplate,
     SyncResponse,
-    build_request,
-    build_response,
+    URLPatternTypes,
 )
 
 
@@ -39,8 +41,7 @@ class BaseMockTransport:
         self.patterns: List[RequestPattern] = []
         self.aliases: Dict[str, RequestPattern] = {}
 
-        self.stats = mock.MagicMock()
-        self.calls: List[Tuple[Request, Optional[Response]]] = []
+        self.calls = CallList()
 
     def clear(self):
         """
@@ -54,22 +55,30 @@ class BaseMockTransport:
         Resets call stats.
         """
         self.calls.clear()
-        self.stats.reset_mock()
+
         for pattern in self.patterns:
-            pattern.stats.reset_mock()
+            pattern.calls.clear()
+
+    @property
+    def stats(self):
+        warn(
+            ".stats property is deprecated. Please, use .calls",
+            category=DeprecationWarning,
+        )
+        return self.calls
 
     def __getitem__(self, alias: str) -> Optional[RequestPattern]:
         return self.aliases.get(alias)
 
     @overload
     def pop(self, alias: str) -> RequestPattern:
-        ...
+        ...  # pragma: nocover
 
     @overload
     def pop(
         self, alias: str, default: DefaultType
     ) -> Union[RequestPattern, DefaultType]:
-        ...
+        ...  # pragma: nocover
 
     def pop(self, alias, default=...):
         """
@@ -89,11 +98,16 @@ class BaseMockTransport:
     def add(
         self,
         method: Union[str, Callable, RequestPattern],
-        url: Optional[Union[str, Pattern]] = None,
+        url: Optional[URLPatternTypes] = None,
+        *,
+        params: Optional[QueryParamTypes] = None,
         status_code: Optional[int] = None,
-        content: Optional[ContentDataTypes] = None,
-        content_type: Optional[str] = None,
         headers: Optional[HeaderTypes] = None,
+        content_type: Optional[str] = None,
+        content: Optional[ContentDataTypes] = None,
+        text: Optional[str] = None,
+        html: Optional[str] = None,
+        json: Optional[JSONTypes] = None,
         pass_through: bool = False,
         alias: Optional[str] = None,
     ) -> RequestPattern:
@@ -105,12 +119,19 @@ class BaseMockTransport:
 
         else:
             response = ResponseTemplate(
-                status_code, headers, content, content_type=content_type
+                status_code,
+                headers=headers,
+                content_type=content_type,
+                content=content,
+                text=text,
+                html=html,
+                json=json,
             )
             pattern = RequestPattern(
                 method,
                 url,
-                response,
+                params=params,
+                response=response,
                 pass_through=pass_through,
                 alias=alias,
                 base_url=self._base_url,
@@ -124,169 +145,224 @@ class BaseMockTransport:
 
     def get(
         self,
-        url: Optional[Union[str, Pattern]] = None,
+        url: Optional[URLPatternTypes] = None,
+        *,
+        params: Optional[QueryParamTypes] = None,
         status_code: Optional[int] = None,
-        content: Optional[ContentDataTypes] = None,
-        content_type: Optional[str] = None,
         headers: Optional[HeaderTypes] = None,
+        content_type: Optional[str] = None,
+        content: Optional[ContentDataTypes] = None,
+        text: Optional[str] = None,
+        html: Optional[str] = None,
+        json: Optional[JSONTypes] = None,
         pass_through: bool = False,
         alias: Optional[str] = None,
     ) -> RequestPattern:
         return self.add(
             "GET",
             url=url,
+            params=params,
             status_code=status_code,
-            content=content,
-            content_type=content_type,
             headers=headers,
+            content_type=content_type,
+            content=content,
+            text=text,
+            html=html,
+            json=json,
             pass_through=pass_through,
             alias=alias,
         )
 
     def post(
         self,
-        url: Optional[Union[str, Pattern]] = None,
+        url: Optional[URLPatternTypes] = None,
+        *,
+        params: Optional[QueryParamTypes] = None,
         status_code: Optional[int] = None,
-        content: Optional[ContentDataTypes] = None,
-        content_type: Optional[str] = None,
         headers: Optional[HeaderTypes] = None,
+        content_type: Optional[str] = None,
+        content: Optional[ContentDataTypes] = None,
+        text: Optional[str] = None,
+        html: Optional[str] = None,
+        json: Optional[JSONTypes] = None,
         pass_through: bool = False,
         alias: Optional[str] = None,
     ) -> RequestPattern:
         return self.add(
             "POST",
             url=url,
+            params=params,
             status_code=status_code,
-            content=content,
-            content_type=content_type,
             headers=headers,
+            content_type=content_type,
+            content=content,
+            text=text,
+            html=html,
+            json=json,
             pass_through=pass_through,
             alias=alias,
         )
 
     def put(
         self,
-        url: Optional[Union[str, Pattern]] = None,
+        url: Optional[URLPatternTypes] = None,
+        *,
+        params: Optional[QueryParamTypes] = None,
         status_code: Optional[int] = None,
-        content: Optional[ContentDataTypes] = None,
-        content_type: Optional[str] = None,
         headers: Optional[HeaderTypes] = None,
+        content_type: Optional[str] = None,
+        content: Optional[ContentDataTypes] = None,
+        text: Optional[str] = None,
+        html: Optional[str] = None,
+        json: Optional[JSONTypes] = None,
         pass_through: bool = False,
         alias: Optional[str] = None,
     ) -> RequestPattern:
         return self.add(
             "PUT",
             url=url,
+            params=params,
             status_code=status_code,
-            content=content,
-            content_type=content_type,
             headers=headers,
+            content_type=content_type,
+            content=content,
+            text=text,
+            html=html,
+            json=json,
             pass_through=pass_through,
             alias=alias,
         )
 
     def patch(
         self,
-        url: Optional[Union[str, Pattern]] = None,
+        url: Optional[URLPatternTypes] = None,
+        *,
+        params: Optional[QueryParamTypes] = None,
         status_code: Optional[int] = None,
-        content: Optional[ContentDataTypes] = None,
-        content_type: Optional[str] = None,
         headers: Optional[HeaderTypes] = None,
+        content_type: Optional[str] = None,
+        content: Optional[ContentDataTypes] = None,
+        text: Optional[str] = None,
+        html: Optional[str] = None,
+        json: Optional[JSONTypes] = None,
         pass_through: bool = False,
         alias: Optional[str] = None,
     ) -> RequestPattern:
         return self.add(
             "PATCH",
             url=url,
+            params=params,
             status_code=status_code,
-            content=content,
-            content_type=content_type,
             headers=headers,
+            content_type=content_type,
+            content=content,
+            text=text,
+            html=html,
+            json=json,
             pass_through=pass_through,
             alias=alias,
         )
 
     def delete(
         self,
-        url: Optional[Union[str, Pattern]] = None,
+        url: Optional[URLPatternTypes] = None,
+        *,
+        params: Optional[QueryParamTypes] = None,
         status_code: Optional[int] = None,
-        content: Optional[ContentDataTypes] = None,
-        content_type: Optional[str] = None,
         headers: Optional[HeaderTypes] = None,
+        content_type: Optional[str] = None,
+        content: Optional[ContentDataTypes] = None,
+        text: Optional[str] = None,
+        html: Optional[str] = None,
+        json: Optional[JSONTypes] = None,
         pass_through: bool = False,
         alias: Optional[str] = None,
     ) -> RequestPattern:
         return self.add(
             "DELETE",
             url=url,
+            params=params,
             status_code=status_code,
-            content=content,
-            content_type=content_type,
             headers=headers,
+            content_type=content_type,
+            content=content,
+            text=text,
+            html=html,
+            json=json,
             pass_through=pass_through,
             alias=alias,
         )
 
     def head(
         self,
-        url: Optional[Union[str, Pattern]] = None,
+        url: Optional[URLPatternTypes] = None,
+        *,
+        params: Optional[QueryParamTypes] = None,
         status_code: Optional[int] = None,
-        content: Optional[ContentDataTypes] = None,
-        content_type: Optional[str] = None,
         headers: Optional[HeaderTypes] = None,
+        content_type: Optional[str] = None,
+        content: Optional[ContentDataTypes] = None,
+        text: Optional[str] = None,
+        html: Optional[str] = None,
+        json: Optional[JSONTypes] = None,
         pass_through: bool = False,
         alias: Optional[str] = None,
     ) -> RequestPattern:
         return self.add(
             "HEAD",
             url=url,
+            params=params,
             status_code=status_code,
-            content=content,
-            content_type=content_type,
             headers=headers,
+            content_type=content_type,
+            content=content,
+            text=text,
+            html=html,
+            json=json,
             pass_through=pass_through,
             alias=alias,
         )
 
     def options(
         self,
-        url: Optional[Union[str, Pattern]] = None,
+        url: Optional[URLPatternTypes] = None,
+        *,
+        params: Optional[QueryParamTypes] = None,
         status_code: Optional[int] = None,
-        content: Optional[ContentDataTypes] = None,
-        content_type: Optional[str] = None,
         headers: Optional[HeaderTypes] = None,
+        content_type: Optional[str] = None,
+        content: Optional[ContentDataTypes] = None,
+        text: Optional[str] = None,
+        html: Optional[str] = None,
+        json: Optional[JSONTypes] = None,
         pass_through: bool = False,
         alias: Optional[str] = None,
     ) -> RequestPattern:
         return self.add(
             "OPTIONS",
             url=url,
+            params=params,
             status_code=status_code,
-            content=content,
-            content_type=content_type,
             headers=headers,
+            content_type=content_type,
+            content=content,
+            text=text,
+            html=html,
+            json=json,
             pass_through=pass_through,
             alias=alias,
         )
 
     def record(
         self,
-        request: Any,
-        response: Optional[Any],
+        request: Request,
+        response: Optional[Response],
         pattern: Optional[RequestPattern] = None,
     ) -> None:
-        request = build_request(request)
-        response = build_response(response, request=request)
-
+        # TODO: Skip recording stats for pass_through requests?
+        call = self.calls.record(request, response)
         if pattern:
-            pattern.stats(request, response)
-
-        self.stats(request, response)
-
-        # Copy stats due to unwanted use of property refs in the high-level api
-        self.calls[:] = (
-            (request, response) for (request, response), _ in self.stats.call_args_list
-        )
+            pattern.calls.append(call)
 
     def assert_all_called(self) -> None:
         assert all(
@@ -337,7 +413,7 @@ class BaseMockTransport:
             # Assert we always get a pattern match, if check is enabled
             assert not self._assert_all_mocked, f"RESPX: {request[1]!r} not mocked!"
 
-            # Auto mock a successfull empty response
+            # Auto mock a successful empty response
             response = ResponseTemplate()
 
         return matched_pattern, request, response
@@ -366,7 +442,9 @@ class BaseMockTransport:
             response = None
             raise
         finally:
-            self.record(request, response, pattern=pattern)
+            self.record(
+                request, response, pattern=pattern
+            )  # pragma: nocover  # python 3.9 bug
 
     async def arequest(
         self,
@@ -392,7 +470,9 @@ class BaseMockTransport:
             response = None
             raise
         finally:
-            self.record(request, response, pattern=pattern)
+            self.record(
+                request, response, pattern=pattern
+            )  # pragma: nocover  # python 3.9 bug
 
     def close(self) -> None:
         if self._assert_all_called:
